package Trickster::DiskJockey;
# $Id: DiskJockey.pm,v 1.2 2002/09/24 01:31:41 miyagawa Exp $

use strict;
use vars qw($Default);
$Default = 'Random';

require Trickster::DiskJockey::Random;

sub new {
    my($class, $config) = @_;
    my $dj_class = $config->get('dj.class') || $Default;
    my $real_class = _load_module($dj_class);
    my $self = bless { config => $config }, $real_class;
    $self->init();
    return $self;
}

sub init { }

sub _load_module {
    my $dj_class = shift;
    eval { require "Trickster/DiskJockey/$dj_class.pm" };
    return "Trickster::DiskJockey::" . ($@ ? 'Random' : $dj_class);
}

1;
__END__

=head1 NAME

Trickster::DiskJockey - DiskJockey factory class

=head1 SYNOPSIS

  use Trickster::DiskJockey;
  my $dj = Trickster::DiskJockey->new($config);
  my $file = $dj->choose_next;

=head1 DESCRIPTION

Trickster::DiskJockey is a factory of various DiskJockey classes,
which define how to choose next songs when nobody make a request.  DJ
class is defined in C<dj.class> key of configuration file, which
default is C<Random>.


=head1 AUTHOR

Tatsuhiko Miyagawa E<lt>miyagawa@bulknews.netE<gt>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 SEE ALSO

L<POE::Component::Trickster::Controller>

=cut





