package Trickster::MediaFile;
# $Id: MediaFile.pm,v 1.2 2002/09/24 06:14:44 miyagawa Exp $

use strict;
use File::Basename;
use MP3::Info;

use constant UNKNOWN => 'unknown';

sub new {
    my($class, $file) = @_;
    my $info = get_mp3info($file) or die "$file: id3 header not found";
    bless {
	file => $file,
	tag  => get_mp3tag($file),
	info => $info,
    }, $class;
}

sub id {
    my $self = shift;
    return (stat($self->file))[1]; # i-node
}

sub file   { shift->{file} }
sub artist { shift->{tag}->{ARTIST} }
sub title  { shift->{tag}->{TITLE} }
sub album  { shift->{tag}->{ALBUM} }

sub is_media_file {
    my $self = shift;
    return -f $self->file;
}

sub info {
    my $self = shift;
    if ($self->{tag}) {
	return sprintf '%s - %s (%s)',
	    $self->artist || UNKNOWN, $self->title || UNKNOWN, $self->album || UNKNOWN;
    } else {
	return basename($self->file);
    }
}

sub bitrate { shift->{info}->{BITRATE} }
sub seconds { shift->{info}->{SECS} }

sub length  {
    my $self = shift;
    return sprintf '%02d:%02d', $self->{info}->{MM}, $self->{info}->{SS};
}

sub delay_for {
    my($self, $bytes) = @_;
    return $bytes * 8 / $self->bitrate / 1_000;
}

1;
