package Trickster::Plugin::Console;
# $Id: Console.pm,v 1.2 2002/09/24 09:29:57 miyagawa Exp $

use strict;
use base qw(Trickster::Plugin);

use POE qw(Wheel::ReadLine);
use DirHandle;
use File::Spec;
use Trickster::MediaFile;

use constant PROMPT => "trickster> ";

sub init {
    my $self = shift;
    POE::Session->create(
	object_states => [
	    $self => [qw(_start _stop _default
			 got_input got_queue got_enqueue got_skip got_error)],
	],
    );
}

sub real_start {
    my($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
    $heap->{wheel} = POE::Wheel::ReadLine->new(InputEvent => 'got_input');
    $heap->{directory} = $self->{config}->get('files.root');
    $heap->{wheel}->get(PROMPT);
}

sub got_input {
    my($self, $kernel, $heap, $input, $ex) = @_[OBJECT, KERNEL, HEAP, ARG0, ARG1];
    if ($input) {
	$heap->{wheel}->addhistory($input);
	$input =~ s/\s*(.*)\s*/$1/;
	if ($input =~ /^ls$/) {
	    my $handle = DirHandle->new($heap->{directory});
	    my $ext = $self->{config}->get('files.ext');
	    for my $file ($handle->read) {
		next if $file =~ /^\./;
		my $real = File::Spec->catfile($heap->{directory}, $file);
		if (-d $real) {
		    $heap->{wheel}->put("* dir *: $file");
		} elsif ($file =~ /$ext$/o) {
		    my $media = Trickster::MediaFile->new($real);
		    $heap->{wheel}->put($media->id . ": " . $self->encode($media->info));
		}
	    }
	}
	elsif ($input =~ /^pwd$/) {
	    $heap->{wheel}->put($heap->{directory});
	}
	elsif ($input =~ /^cd (.*)/) {
	    my $dir = File::Spec->canonpath(File::Spec->catdir($heap->{directory}, $1));
	    if (-d $dir) {
		$heap->{directory} = $dir;
	    } else {
		$heap->{wheel}->put("$dir: no such directory");
	    }
	}
	elsif ($input =~ /^(qu|ex)it$/) {
	    _shutdown($kernel, $heap);
	    return;
	}
	elsif ($input =~ /^queue (.*)$/) {
	    $kernel->post(player => enqueue => $1);
	    return;
	}
	elsif ($input =~ /^skip$/) {
	    $kernel->post(player => 'skip');
	    return;
	}
	elsif ($input =~ /^queue$/) {
	    $kernel->post(player => get_queue => 'got_queue');
	    return;
	}
	else {
	    $heap->{wheel}->put("$input: command not found.");
	}
    }
    if ($ex && $ex eq 'interrupt') {
	_shutdown($kernel, $heap);
	return;
    }
    $heap->{wheel}->get(PROMPT);
}

sub got_queue {
    my($self, $kernel, $heap, $curr, $files) = @_[OBJECT, KERNEL, HEAP, ARG0, ARG1];
    $heap->{wheel}->put($self->encode($curr->id . ": " . $curr->info . " <= now playing"));
    for my $file (@$files) {
	$heap->{wheel}->put($self->encode($file->id . ": " . $file->info));
    }
    $heap->{wheel}->get(PROMPT);
}

sub got_enqueue {
    $_[HEAP]->{wheel}->put("queued ". $_[ARG0]->id);
    $_[HEAP]->{wheel}->get(PROMPT);
}

sub got_skip {
    $_[HEAP]->{wheel}->put("skipped");
    $_[HEAP]->{wheel}->get(PROMPT);
}

sub got_error {
    $_[HEAP]->{wheel}->put("server error: " . $_[ARG0]);
}

sub _shutdown {
    my($kernel, $heap) = @_;
    delete $heap->{wheel};
    $kernel->post(player => 'shutdown');
    $kernel->post(player => 'unregister');
}

sub encode {
    my($self, $str) = @_;
    return eval $self->{config}->get('plugin.console.encode');
}

1;
