package Trickster::Plugin::IRCBot;
# $Id: IRCBot.pm,v 1.1 2002/09/24 09:29:57 miyagawa Exp $

use strict;
use base qw(Trickster::Plugin);
use POE qw(Component::IRC);
use DirHandle;
use File::Spec;
use Trickster::MediaFile;
use URI;

sub init {
    my $self = shift;
    POE::Session->create(
	object_states => [
	    $self => [ qw(_start _stop _default
			  irc_001 irc_public irc_msg
			  got_start got_enqueue
		      ) ],
	],
    );
}

sub real_start {
    my($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
    POE::Component::IRC->new("bot");
    $heap->{me} = $self->{config}->get('plugin.bot.nick');
    $heap->{channels} = [ split /\s+/, $self->{config}->get('plugin.bot.channels') ];
    $kernel->post(bot => register => qw(001 msg public));
    $kernel->post(bot => connect => {
	Nick => $heap->{me},
	Server => $self->{config}->get('plugin.bot.server'),
	Port => $self->{config}->get('plugin.bot.port') || 6667,
	Username => $self->{config}->get('plugin.bot.username'),
	Ircname => __PACKAGE__,
    });
}

sub irc_001 {
    my($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
    $kernel->post(bot => join => $_) for @{$heap->{channels}};
}

sub irc_public {
    my($self, $kernel, $heap, $nick, $chan, $text) = @_[OBJECT, KERNEL, HEAP, ARG0 .. $#_];
    $text =~ s/^$heap->{me}\s*[,:]\s*//;
    for ($text) {
	/^help/ and do {
	    $kernel->post(bot => privmsg => $chan, "help | queue [id] | skip | url | listeners");
	};
	/^queue\s*(\S*)$/ and do {
	    if ($1) {
		$kernel->post(player => enqueue => $1);
	    } else {
		$kernel->post(
		    player => get_queue => sub {
			my($curr, $files) = @_;
			$kernel->post(bot => privmsg => $heap->{channels} =>
					  $self->encode("Now playing: " . $curr->info));
			for my $f (@$files) {
			    $kernel->post(bot => privmsg => $heap->{channels} =>
					      $self->encode($f->info));
			}
		    },
		);
	    }
	};
	/^skip$/ and do {
	    $kernel->post(player => 'skip');
	};
	/^url$/ and do {
	    $kernel->post(bot => privmsg => $chan, $self->streaming_url);
	};
	/^listeners$/ and do {
	    $kernel->post(
		player => get_listeners => sub {
		    my @listerners = @_;
		    $kernel->post(bot => privmsg => $chan =>
				      join(" / ", map sprintf("%s (%s)", $_->addr, $_->agent), @listerners));
		},
	    )
	};
    }
}

sub got_start {
    my($self, $kernel, $heap, $file) = @_[OBJECT, KERNEL, HEAP, ARG0];
    $kernel->post(bot => privmsg => $heap->{channels}, $self->encode("start playing: " . $file->info));
}

sub got_enqueue {
    my($self, $kernel, $heap, $file) = @_[OBJECT, KERNEL, HEAP, ARG0];
    $kernel->post(bot => privmsg => $heap->{channels}, $self->encode("queued: " . $file->info));
}

sub encode {
    my($self, $str) = @_;
    eval $self->{config}->get('plugin.bot.encode') || $str;
}

sub streaming_url {
    my $self = shift;
    my $uri = URI->new;
    $uri->scheme('http');
    $uri->host($self->{config}->get('streaming.hostname'));
    $uri->port($self->{config}->get('streaming.port'));
    $uri->path('/');
    return $uri->as_string;
}

sub irc_msg { }

1;
